import sys

versionString = '%s: version 1.3' % sys.argv[0]
print versionString

try:
    import psyco
    psyco.full()
except:
    print 'psyco not running'

from som import SOM
from mapGraphics import saveMapPNG, coordTranslate

if len(sys.argv) < 6:
    print 'usage: python %s somfile1 scorefile1 somefile2 scorefile2 outprefix [-verbose] [-radius num]' % sys.argv[0]
    sys.exit(0)

doVerbose = False

somfile1 = sys.argv[1]
scorefile1 = sys.argv[2]

somfile2 = sys.argv[3]
scorefile2 = sys.argv[4]

outprefix = sys.argv[5]

if '-verbose' in sys.argv:
    doVerbose = True

maxRadius = 2
if '-radius' in sys.argv:
    maxRadius = int(sys.argv[sys.argv.index('-radius') + 1])

mysom1 = SOM(initialFile=somfile1)
if doVerbose:
    print "read SOM1..."

winnerDict1 = mysom1.readScoreFile(scorefile1)

mysom2 = SOM(initialFile=somfile2)
if doVerbose:
    print "read SOM2..."

winnerDict2 = mysom2.readScoreFile(scorefile2)

if doVerbose:
    print "computing SOM2 rev Dict..."

revDict2 = {}
for (row, col) in winnerDict2:
    for regionID in winnerDict2[row, col]:
        revDict2[regionID] = (row, col)

if doVerbose:
    print "computing neighborhoodDict of radius = %d" % maxRadius
neighborDict = mysom2.buildNeighborDict(maxRadius)

somsize2 = len(mysom2.units)

outmapImmediate = mysom1.newMap(numDim=0)
outmapNeighbor  = mysom1.newMap(numDim=0)

outfile = open(outprefix + '.txt', 'w')
outfile.write('#som1\tregions\tsom2\tbest\tneighbors\ttotal\n')
for (row1, col1) in mysom1.units:
    localMap = {}
    numRegions = len(winnerDict1[row1, col1])
    if numRegions == 0:
        numRegions = 1
    for regionID in winnerDict1[row1, col1]:
        (row2, col2) = revDict2[regionID]
        try:
            localMap[row2, col2] += 1
        except:
            localMap[row2, col2] = 1
    bestRow = -1
    bestCol = -1
    bestScore = 0
    numUnits = len(localMap)
    for (row2, col2) in localMap:
        unitCount = localMap[row2, col2]
        if unitCount > bestScore:
            bestRow = row2
            bestCol = col2
            bestScore = unitCount
    neighborSum = 0.
    for radius in range(1, maxRadius+1):
        if bestRow < 0 or bestCol < 0:
            continue
        for (rowi, coli) in neighborDict[bestRow, bestCol][radius]:
            try:
                neighborSum += localMap[rowi, coli]
            except:
                pass
    
    bestFrac = float(bestScore) / numRegions
    neighborFrac = float(neighborSum) / numRegions
    totalFrac = bestFrac + neighborFrac
    outline = 'unit_%d_%d\t%d\tunit_%d_%d\t%.2f\t%.2f\t%.2f\n' % (row1, col1, numRegions, bestRow, bestCol, bestFrac, neighborFrac, totalFrac)
    outfile.write(outline)
    outmapImmediate[row1][col1] = neighborFrac
    outmapNeighbor[row1][col1] = totalFrac
outfile.close()

saveMapPNG(outmapImmediate, outprefix + '.im', grid=mysom1.gridType)
saveMapPNG(outmapNeighbor, outprefix + '.nb', grid=mysom1.gridType)

mysom1.saveMap(outmapImmediate, outprefix + '.im.map')
mysom1.saveMap(outmapNeighbor, outprefix + '.nb.map')
