import sys

versionString = '%s: version 1.4' % sys.argv[0]
print versionString

try:
    import psyco
    psyco.full()
except:
    print 'psyco not running'

from som import SOM
from mapGraphics import saveMapPNG, coordTranslate

if len(sys.argv) < 6:
    print "usage: python %s somfile inmap1 operation inmap2 outprefix [-title string] [-cluster cluster] [-vmax value] [-vmin value] [-correlate]" % sys.argv[0]
    sys.exit(0)

somfile = sys.argv[1]
inmapfile1 = sys.argv[2]
operation = sys.argv[3]
inmapfile2 = sys.argv[4]
outfileprefix = sys.argv[5]

doCoords = False

outfiletitle = '%s %s %s' % (inmapfile1, operation, inmapfile2)
if '-title' in sys.argv:
    outfiletitle = sys.argv[sys.argv.index('-title') + 1]

maxValue = 0
if '-vmax' in sys.argv:
    maxValue = float(sys.argv[sys.argv.index('-vmax') + 1])

minValue = 0
if '-vmin' in sys.argv:
    minValue = float(sys.argv[sys.argv.index('-vmin') + 1])

doCorrelation = False
if '-correlate' in sys.argv:
    doCorrelation = True

selectUnits = False
selectList = []
if '-cluster' in sys.argv:
    selectUnits = True
    clusterfile = open(sys.argv[sys.argv.index('-cluster') + 1])
    for line in clusterfile:
        fields = line.strip().split('\t')
        unitID = fields[0]
        uCoords = unitID.split('_')
        uRow = int(uCoords[1])
        uCol = int(uCoords[2])
        selectList.append((uRow, uCol))
    
mysom = SOM(initialFile=somfile)

xmax = mysom.outCols
ymax = mysom.outRows
onePercent = int(xmax * ymax / 100.)
if onePercent < 1:
    onePercent = 1

inmap1 = mysom.readMap(inmapfile1)
inmap2 = mysom.readMap(inmapfile2)
outmap = mysom.newMap()

mapMinValue = minValue
mapMaxValue = maxValue
unitValueList = []

for (row, col) in mysom.units:
    if operation == 'SUB':
        outmap[row][col] = inmap1[row][col] - inmap2[row][col]
    elif operation == 'ADD':
        outmap[row][col] = inmap1[row][col] + inmap2[row][col]
    elif operation == 'MULT':
        outmap[row][col] = inmap1[row][col] * inmap2[row][col]
    elif operation == 'MIN':
        outmap[row][col] = min(inmap1[row][col],inmap2[row][col])
    elif operation == 'MAX':
        outmap[row][col] = max(inmap1[row][col],inmap2[row][col])                
    if outmap[row][col] > 0:
        unitValueList.append(outmap[row][col])
    else:
        unitValueList.append(0)

mysom.saveMap(outmap, outfileprefix + '.map')
#outmap.reverse()
unitValueList.sort()
if mapMinValue == 0:
    mapMinValue = unitValueList[onePercent]
if mapMaxValue == 0:
    mapMaxValue = unitValueList[-1 * onePercent]

if doCorrelation and operation == 'MIN':
    corr = mysom.mapPearsonCorrelation(inmap1, inmap2)
    outfiletitle += ' corr: %.3f' % corr

saveMapPNG(outmap, outfileprefix, outfiletitle, minVal=mapMinValue, maxVal=mapMaxValue, grid=mysom.gridType, highlightList=selectList)

