import sys, string, math
from operator import itemgetter

versionString = '%s: version 2.0' % sys.argv[0]
print versionString

try:
    import psyco
    psyco.full()
except:
    print "psyco not running"

from som import SOM
from mapGraphics import saveMapPNG, coordTranslate

if len(sys.argv) < 4:
    print "usage: python %s somfile scorefile outprefix [-cluster clusterfile] [-map mapfile] [-top num] [-singlefile] [-translate dx,dy] [-full] [-regions] [-consolidate]" % sys.argv[0]
    print "\twhere -consolidate assumes that region identifiers are of the form chr:start-stop"
    sys.exit(0)

somfile = sys.argv[1]
scorefile = sys.argv[2]
outprefix = sys.argv[3]

dx = 0
dy = 0
idField = 1
doFull = False
if '-full' in sys.argv:
    doFull = True

doCoords = False
if '-translate' in sys.argv:
    coords = sys.argv[sys.argv.index('-translate') + 1]
    (dx, dy) = coords.split(',')
    dx = int(dx)
    dy = int(dy)
    doCoords = True

selectUnits = False
selectList = []
if '-cluster' in sys.argv:
    selectUnits = True
    clusterfile = open(sys.argv[sys.argv.index('-cluster') + 1])
    for line in clusterfile:
        fields = line.strip().split('\t')
        unitID = fields[0]
        uCoords = unitID.split('_')
        uRow = int(uCoords[1])
        uCol = int(uCoords[2])
        selectList.append((uRow, uCol))

doMultiFile = True
if '-singlefile' in sys.argv:
    doMultiFile = False
    
keepTop = -1
if '-top' in sys.argv:
    keepTop = int(sys.argv[sys.argv.index('-top') + 1])

doMap = False
if '-map' in sys.argv:
    selectUnits = True
    doMap = True
    mapfile = sys.argv[sys.argv.index('-map') + 1]
    
doRegions = False
if '-regions' in sys.argv:
    doRegions = True
    if doFull:
        print "-consolidate over-rides -full  - reporting only regions"
        doFull = False

consolidate = False
consRadius = 2
if '-consolidate' in sys.argv:
    consolidate = True
    doRegions = True
    if doFull:
        print "-consolidate over-rides -full  - reporting only regions"
        doFull = False

mysom = SOM(initialFile=somfile)
units = mysom.getUnits()

if doMap:
    inputmap = mysom.readMap(mapfile)
    localList = []
    for (row, col) in units:
        if inputmap[row][col] > 0:
            localList.append((inputmap[row][col], (row, col)))
    localList.sort()
    localList.reverse()
    for (score, aUnit) in localList[:keepTop]:
        selectList.append(aUnit)
    
xmax = mysom.outCols
ymax = mysom.outRows
onePercent = int(xmax * ymax / 100.)
if onePercent < 1:
    onePercent = 1

if consolidate:
    idField = 0
    winnerDict = mysom.readScoreFile(scorefile)
else:
    idField = 1
    winnerDict = mysom.readScoreFile(scorefile, full=True)

if not doMultiFile:
    outfilename = outprefix + '.comb.txt'
    outfile = open(outfilename,'w')

outmap = mysom.newMap(numDim=0)
for (row, col) in units:
    if selectUnits and (row, col) not in selectList:
        continue
    workingList = winnerDict[row, col]
    if doMultiFile:
        outfilename = '%s_%d_%d.unit' % (outprefix, row, col)
        outfile = open(outfilename,'w')
    if doRegions:
        workingList = []
        for line in winnerDict[row, col]:
            fields = line.strip().split()
            (chrom,coords) = fields[idField].split(':')
            (start,stop) = coords.split('-')
            if consolidate:
                workingList.append((chrom, int(start), int(stop)))
            else:
                workingList.append((chrom, int(start), int(stop), fields[0]))
        outfile.write('#regionID\tchrom\tstart\tstop\t')
        if consolidate:
            outfile.write('origRegions')
        else:
            outfile.write('score')
    if doFull:
        outfile.write('#score\tregionID\tother_columns\n')
    else:
        outfile.write('\n')
    
    outlineList = []
    currentStart = -1
    currentStop  = -1
    currentChrom = ''
    consString = ''
    for data in sorted(workingList):
        if consolidate:
            (newChrom, newStart, newStop) = data
            if abs(newStart - currentStop) > consRadius or newChrom != currentChrom:
                if currentChrom != '':
                    outline = '%s:%s-%s\t%s\t%s\t%s\t%s\n' % (currentChrom, currentStart, currentStop, currentChrom, currentStart, currentStop, consString[1:])
                    outlineList.append(outline)
                currentChrom = newChrom
                currentStart = newStart
                consString = ''
                outline = ''
            currentStop  = newStop
            consString += ',%s:%s-%s' % (newChrom, newStart, newStop)
            outmap[row][col] += abs(int(newStop) - int(newStart)) + 1
        elif doRegions:
            (chrom, start, stop, outval) = data
            outline = '%s:%s-%s\t%s\t%s\t%s\t%s' % (chrom, start, stop, chrom, start, stop, outval)
            #if doFull:
            #    outline += string.join(fields[2:],'\t')
            outline += '\n'
            outmap[row][col] += abs(int(stop) - int(start)) + 1
        else:
            if doFull:
                outline = data
            else:
                fields = data.split()
                outline = '%s\t%s\n' % (fields[0], fields[1])
        if len(outline) > 0:
            outlineList.append(outline)
    if consString != '':
        outline = '%s:%s-%s\t%s\t%s\t%s\t%s\n' % (currentChrom, currentStart, currentStop, currentChrom, currentStart, currentStop, consString[1:])
        outlineList.append(outline)
    
    outlineList.sort()
    for line in outlineList:
        outfile.write(line)
    if doRegions:
        outfile.write('# nucleotide count: %d bp\n' % outmap[row][col])
    if doMultiFile:
        outfile.close()

if not doMultiFile:
    outfile.close()

if doRegions:
    for (row, col) in units:
        if outmap[row][col] == 0:
            outmap[row][col] = 1
        outmap[row][col] = math.log(outmap[row][col]/1000., 10)
    outfiletitle = 'log10(nucleotide kb count)'
    mysom.saveMap(outmap, outprefix + 'nuc.map')
    saveMapPNG(outmap, outprefix + '.nuc', outfiletitle, minVal=-2, grid=mysom.gridType)
