
import matplotlib, copy
matplotlib.use('Agg')
from pylab import *
from matplotlib.patches import Circle, RegularPolygon
from matplotlib.collections import PatchCollection

pi_value = 3.14159

def saveMapPNG(regArray, fileprefix, figTitle='', minVal=0, maxVal=0, grid='rect', highlightList = [], highlightColor='#FFFFFF', alpha=0.75, mapType='fraction'):
    height = len(regArray)
    width = len(regArray[0])
    if grid == 'hex':
        # from Shirley's code
        fig = figure(figsize=(6, 4))
        #theArray = convertHexToRect(regArray)
        patches = []
        patches2 = []
        ax = fig.add_subplot(1,1,1,aspect='equal')
        subplots_adjust(0.05, 0., 1., 1., 0, 0)
        x = arange(0., height * width)
        highlightArray = arange(0., len(highlightList))
        highlightIndex = 0
        highlightEdgeDict = {}
        for row in range(height):
            for col in range(width):
                poly = RegularPolygon((col+.5*(row%2),sqrt(.75)*(height-row-1)),6,.53,0)
                patches.append(poly)
                theVal = regArray[row][col]
                if maxVal > 0 and theVal > maxVal:
                    theVal = maxVal
                if theVal < minVal:
                    theVal = minVal
                x[row*width + col] = theVal
                if (row, col) in highlightList:
                    highlightIndex += 1
                    #start1 = 5/6.
                    #start2 = 2/3.
                    start1 = 0.
                    start2 = 1/6.
                    vm = 1./sqrt(3.)
                    #vm = 1
                    x0 = col+.5*(row%2)
                    y0 = sqrt(.75)*(height-row-1)
                    for count in range(6):
                        x1 = round(x0 + vm * sin(2 * pi * (start1 - count/6.)),2)
                        y1 = round(y0 + vm * cos(2 * pi * (start1 - count/6.)),2)
                        x2 = round(x0 + vm * sin(2 * pi * (start2 - count/6.)),2)
                        y2 = round(y0 + vm * cos(2 * pi * (start2 - count/6.)),2)
                        line = ((x1, y1),(x2, y2))
                        revline = ((x2, y2),(x1, y1))
                        if line in highlightEdgeDict:
                            highlightEdgeDict[line] += 1
                        elif revline in highlightEdgeDict:
                            highlightEdgeDict[revline] += 1
                        else:
                            # we've seen this line once so far
                            highlightEdgeDict[line] = 1
                    
	ax.set_ylim(-0.6,sqrt(.75)*(height-1)+0.6)
        ax.set_xlim(-0.6,width)
        ax.set_xticks([0,width-0.5])
        ax.set_yticks([0,sqrt(.75)*(height-1)])
        ax.set_xticklabels(('0',str(width -1)))
        ax.set_yticklabels((str(height -1), '0'))
        if mapType == 'count':
            p = PatchCollection(patches, cmap=matplotlib.cm.Greens)
        else:
            p = PatchCollection(patches, cmap=matplotlib.cm.RdYlBu_r)
        
	p.set_edgecolor('face')
	p.set_array(x)
	ax.add_collection(p)
        if highlightIndex > 0:
            for line in highlightEdgeDict:
                if highlightEdgeDict[line] > 1:
                    continue
                (x1, y1) = line[0]
                (x2, y2) = line[1]
                if mapType == 'count':
                    plot([x1, x2], [y1, y2], linewidth=2, color='orange')
                else:
                    plot([x1, x2], [y1, y2], linewidth=2, color='black')
        cb = colorbar(p,shrink=.6)
    else:
        figure(figsize=(3, 2))
        theArray = regArray
        Z = array(theArray)
        if maxVal > 0:
            C = pcolor(Z, vmin=minVal, vmax=maxVal, edgecolors='None')
        else:
            C = pcolor(Z, vmin=minVal, edgecolors='None')
        
        C.set_linewidth(0)
        D = colorbar(orientation="vertical", drawedges=False)
        C.xticks([1, width - 1])
        C.yticks([0, height - 1])
        
    if len(figTitle) > 0:
        title(figTitle)
    savefig(fileprefix + '.png', dpi=250)
    show()
    close()
    
def coordTranslate(xorig, yorig, dx, dy, xmax, ymax):
    newx = (xorig + dx) % xmax
    newy = (yorig + dy) % ymax
    
    return newx, newy
    
def convertHexToRect(theArray):
    newArray = []
    numRows = len(theArray)
    numCols = len(theArray[0])
    newCols = (numCols + 1) * 2
    for row in range(numRows):
        newrow1 = [0.] * newCols
        newrow2 = [0.] * newCols
        if row % 2 == 0:
            spacer = 0
        else:
            spacer = 1
        
        for aCol in range(0, 2 * numCols, 2):
            oCol = aCol/2
            newrow1[spacer + aCol]     = theArray[row][oCol]
            newrow1[spacer + aCol + 1] = theArray[row][oCol]
            newrow2[spacer + aCol]     = theArray[row][oCol]
            newrow2[spacer + aCol + 1] = theArray[row][oCol]
        newArray.append(newrow1)
        newArray.append(newrow2)
    
    return newArray

    