import sys, string, os

versionString = '%s: version 1.5' % sys.argv[0]
print versionString

try:
    import psyco
    psyco.full()
except:
    print "psyco not running"

from som import SOM
from mapGraphics import saveMapPNG, coordTranslate

if len(sys.argv) < 4:
    print "usage: python %s somfile godir goprefix outprefix  [-markmap map] [-marktop units] [-savemap]" % sys.argv[0]
    sys.exit(0)

keepTop = 0
markmapfile = ''
if '-markmap' in sys.argv:
    markmapfile = sys.argv[sys.argv.index('-markmap') + 1]
    if '-marktop' in sys.argv:
        keepTop = int(sys.argv[sys.argv.index('-marktop') + 1])

saveMap = False
if '-savemap' in sys.argv:
    saveMap = True

somfile = sys.argv[1]
godir = sys.argv[2]
goprefix = sys.argv[3]
outprefix = sys.argv[4]

goprefixlen = len(goprefix)

mysom = SOM(initialFile=somfile)
units = mysom.getUnits()

xmax = mysom.outCols
ymax = mysom.outRows
onePercent = int(xmax * ymax / 100.)
if onePercent < 1:
    onePercent = 1

selectList = []
if markmapfile != '': 
    if keepTop == 0:
        keepTop = onePercent
    overlaymap = mysom.readMap(markmapfile)
    selectList = mysom.topMapUnits(overlaymap, keepTop)

allfiles = os.listdir(godir)

enrichedGO = {}
goDesc = {}
print "reading gosig files"
for filename in allfiles:
    if goprefix == filename[:goprefixlen]:
        if '.gosig' in filename:
            (front,back) = filename.split('.')
            namefields = front.split('_')
            row = int(namefields[-2])
            col = int(namefields[-1])
            infile = open(filename)
            for line in infile:
                if 'depleted' in line:
                    continue
                fields = line.split()
                #print fields
                GOID = fields[0]
                count = int(fields[1])
                try:
                    enrichedGO[GOID].append((row, col, count))
                except:
                    enrichedGO[GOID] = [(row, col, count)]
                    goDesc[GOID] = string.join(fields[8:],' ')
            infile.close()

gofileDict = {}

summaryfile = open(outprefix + '.summary','w')
print "Creating maps"
combinedmap = mysom.newMap(numDim=0)
for GOID in enrichedGO:
    outmap = mysom.newMap(numDim=0)
    outfiletitle = GOID + ' ' + goDesc[GOID]
    unitCount = 0
    for (row, col, count) in enrichedGO[GOID]:
        outmap[row][col] = count
        combinedmap[row][col] += 1
        unitCount += 1
    outfileprefix = outprefix + '.GO.' + GOID[3:]
    saveMapPNG(outmap, outfileprefix, outfiletitle, grid=mysom.gridType, highlightList=selectList, mapType='count')
    if saveMap:
        mysom.saveMap(outmap, outfileprefix + '.map')
    summaryLine = '%d\t%s\t%s' % (unitCount, outfileprefix, goDesc[GOID])
    print summaryLine
    summaryfile.write(summaryLine + '\n')
saveMapPNG(combinedmap, outprefix + '.summary', '# enriched GO terms per unit', grid=mysom.gridType, highlightList=selectList, mapType='count')
if saveMap:
    mysom.saveMap(combinedmap, outprefix + 'summary.map')
summaryfile.close()

