import sys

versionString = '%s: version 1.8' % sys.argv[0]
print versionString

try:
    import psyco
    psyco.full()
except:
    print 'psyco not running'

from som import SOM
from mapGraphics import saveMapPNG, coordTranslate

if len(sys.argv) < 5:
    print "usage: python %s somfile startmap overlaymap outprefix [-top count] [-threshold num] [-title string] [-vmax value] [-vmin value] [-correlate] [-savemap] [-count]" % sys.argv[0]
    sys.exit(0)

somfile = sys.argv[1]
startmapfile = sys.argv[2]
overlaymapfile = sys.argv[3]
outfileprefix = sys.argv[4]

doCoords = False
doThreshold = False

keepTop = 0
if '-top' in sys.argv:
    keepTop = int(sys.argv[sys.argv.index('-top') + 1])

threshold = 0.
passThreshold = 0
if '-threshold' in sys.argv:
    threshold = float(sys.argv[sys.argv.index('-threshold') + 1])
    doThreshold = True

outfiletitle = '%s' % (startmapfile)
if '-title' in sys.argv:
    outfiletitle = sys.argv[sys.argv.index('-title') + 1]

maxValue = 0
if '-vmax' in sys.argv:
    maxValue = float(sys.argv[sys.argv.index('-vmax') + 1])

minValue = 0
if '-vmin' in sys.argv:
    minValue = float(sys.argv[sys.argv.index('-vmin') + 1])

doCorrelation = False
if '-correlate' in sys.argv:
    doCorrelation = True

saveMap = False
if '-savemap' in sys.argv:
    saveMap = True

mType='fraction'
if '-count' in sys.argv:
    mType='count'

selectUnits = True
selectList = []
if '-cluster' in sys.argv:
    selectUnits = True
    clusterfile = open(sys.argv[sys.argv.index('-cluster') + 1])
    for line in clusterfile:
        fields = line.strip().split('\t')
        unitID = fields[0]
        uCoords = unitID.split('_')
        uRow = int(uCoords[1])
        uCol = int(uCoords[2])
        selectList.append((uRow, uCol))
    
mysom = SOM(initialFile=somfile)

xmax = mysom.outCols
ymax = mysom.outRows
onePercent = int(xmax * ymax / 100.)
if onePercent < 1:
    onePercent = 1

if keepTop == 0:
    keepTop = onePercent

startmap = mysom.readMap(startmapfile)
overlaymap = mysom.readMap(overlaymapfile)
outmap = mysom.newMap(numDim=0)

mapMinValue = minValue
mapMaxValue = maxValue
unitValueList = []
for (row, col) in mysom.units:
    if startmap[row][col] > 0:
        unitValueList.append(startmap[row][col])
    else:
        unitValueList.append(0)
    if doThreshold and overlaymap[row][col] > threshold:
        passThreshold += 1

#outmap.reverse()
unitValueList.sort()
if mapMinValue == 0:
    mapMinValue = unitValueList[onePercent]
if mapMaxValue == 0:
    mapMaxValue = unitValueList[-1 * onePercent]

if doThreshold:
    keepTop = passThreshold
    print "%d units passed threshold %f will be marked" % (passThreshold, threshold)

selectList = mysom.topMapUnits(overlaymap, keepTop)
if saveMap:
    for (row, col) in selectList:
        outmap[row][col] = startmap[row][col]
    mysom.saveMap(outmap, outfileprefix + '.map')

if doCorrelation:
    corr = mysom.mapPearsonCorrelation(startmap, overlaymap)
    outfiletitle += ' (corr: %.3f)' % corr

saveMapPNG(startmap, outfileprefix, outfiletitle, minVal=mapMinValue, maxVal=mapMaxValue, grid=mysom.gridType, highlightList=selectList, mapType=mType)

