import sys, string

from commoncode import getMergedRegions

versionString = '%s: version 1.5' % sys.argv[0]
print versionString

try:
    import psyco
    psyco.full()
except:
    print "psyco not running"

from som import SOM
from mapGraphics import saveMapPNG, coordTranslate

if len(sys.argv) < 4:
    print "usage: python %s somfile scorefile regionfile outprefix [-count] [-perKB] [-merge bp] [-title atitle] [-vmax value] [-vmin value] [-usePeak] [-verbose] [-translate dx,dy]" % sys.argv[0]
    sys.exit(0)

somfile = sys.argv[1]
scorefile = sys.argv[2]
regionfile = sys.argv[3]
outprefix = sys.argv[4]

doNormalization = True
perKB = False
if '-count' in sys.argv:
    print "Will report count data"
    doNormalization = False
else:
    if '-perKB' in sys.argv:
        perKB = True
        print "Will normalize to unit region count and aggregate sequence in kb"
    else:
        print "Will normalize to unit region count"

mergeDist = 0
merging = False
if '-merge' in sys.argv:
    mergeDist = int(sys.argv[sys.argv.index('-merge') + 1])
    merging = True

maxValue = 0
if '-vmax' in sys.argv:
    maxValue = float(sys.argv[sys.argv.index('-vmax') + 1])

minValue = 0
if '-vmin' in sys.argv:
    minValue = float(sys.argv[sys.argv.index('-vmin') + 1])

doCoords = False
if '-translate' in sys.argv:
    coords = sys.argv[sys.argv.index('-translate') + 1]
    (dx, dy) = coords.split(',')
    dx = int(dx)
    dy = int(dy)
    doCoords = True

outfiletitle = ''
if '-title' in sys.argv:
    outfiletitle = sys.argv[sys.argv.index('-title') + 1]

usePeak = False
if '-usePeak' in sys.argv:
    print "Will use chip-seq peak summits"
    usePeak = True

doVerbose = False
if '-verbose' in sys.argv:
    doVerbose = True

mysom = SOM(initialFile=somfile)

xmax = mysom.outCols
ymax = mysom.outRows
onePercent = int(xmax * ymax / 100.)
if onePercent < 1:
    onePercent = 1

winnerDict = mysom.readScoreFile(scorefile)
regionDict = getMergedRegions(regionfile, maxDist = mergeDist,  minHits=-1, keepLabel = True, fullChrom=True, verbose = True, doMerge=merging, keepPeak=usePeak)
regionChroms = regionDict.keys()
regionChroms.sort()
somRegionToUnit = {}
for chrom in regionChroms:
    somRegionToUnit[chrom] = []

outmap = mysom.newMap(numDim=1)
winnerCountDict = {}
winnerKBDict = {}
for unit in winnerDict:
    outmap[unit[0]][unit[1]] = 0.
    if doNormalization:
        winnerCountDict[unit] = float(len(winnerDict[unit]))
    kbSize = 0.
    for region in winnerDict[unit]:
        (chrom, coord) = region.split(':')
        (start, stop) = coord.split('-')
        if perKB:
            kbSize += abs(int(stop) - int(start))/1000.
        if chrom not in regionChroms:
            continue
        somRegionToUnit[chrom].append((int(start), unit))
    if perKB:
        winnerKBDict[unit] = kbSize

for chrom in regionChroms:
    somRegionToUnit[chrom].sort()

for chrom in regionChroms:
    partIndex = 0
    if len(somRegionToUnit[chrom]) < 1:
        continue
    for region in regionDict[chrom]:
        if usePeak:
            (label, start, stop, length, peakPos, peakHeight) = region
            # convert peak into a 1-bp region
            start = peakPos
            stop = peakPos + 1
        else:
            (label, start, stop, length) = region
        inRegion = False
        if start > somRegionToUnit[chrom][-1][0]:
            (row, col) = somRegionToUnit[chrom][-1][1]
            outmap[row][col] += 1.
            if doVerbose:
                print "%s: %d %d" % (label, row, col)
            continue
        elif start < somRegionToUnit[chrom][0][0]:
            continue
        else:
            counter = 0
            prevUnit = somRegionToUnit[chrom][partIndex - 1][1]
            for (pstart, unit) in somRegionToUnit[chrom][partIndex:]:
                if pstart < start:
                    counter += 1
                    continue
                (prow, pcol) = prevUnit
                outmap[prow][pcol] += 1.
                if doVerbose:
                    print "%s: %d %d" % (label, prow, pcol)
                if pstart > stop:
                    break
                counter += 1
                prevUnit = unit
            partIndex += counter

if perKB:
    for unit in winnerKBDict:
        try:
            outmap[unit[0]][unit[1]] /= winnerKBDict[unit]
        except:
            pass

if doNormalization:
    unitValueList = []
    for unit in winnerCountDict:
        try:
            outmap[unit[0]][unit[1]] /= winnerCountDict[unit]
            unitValueList.append(outmap[unit[0]][unit[1]])
        except:
            pass
    unitValueList.sort()
    if minValue == 0:
        minValue = unitValueList[onePercent]
    if maxValue == 0:
        maxValue = unitValueList[-1 * onePercent]        

mysom.saveMap(outmap, outprefix + '.map')
saveMapPNG(outmap, outprefix, outfiletitle, minVal=minValue, maxVal=maxValue, grid=mysom.gridType)

