import sys

versionString = '%s: version 1.8' % sys.argv[0]
print versionString

import matplotlib
matplotlib.use('Agg')

from pylab import *

try:
    import psyco
    psyco.full()
except:
    print "psyco not running"

from som import SOM

if len(sys.argv) < 5:
    print "usage: python %s somfile scorefile count startmap outprefix [-rank] [-header datafile] [-threshold num] [-recenter] [-compact] [-report]" % sys.argv[0]
    sys.exit(0)

somfile = sys.argv[1]
scorefile = sys.argv[2]
count = int(sys.argv[3])
startmapfile = sys.argv[4]
outprefix = sys.argv[5]
outfilename = outprefix + '.stats'
outimagename = outprefix + '.png'

length = 10
width = 7

useRank = False
if '-rank' in sys.argv:
    useRank = True

putHeader = False
datafile = ''
if '-header' in sys.argv:
    putHeader = True
    datafile = sys.argv[sys.argv.index('-header') + 1]

doThreshold = False
threshold = 0.
if '-threshold' in sys.argv:
    threshold = float(sys.argv[sys.argv.index('-threshold') + 1])
    doThreshold = True
    print "using threshold - will ignore count limit"

doRecenter = False
if '-recenter' in sys.argv:
    doRecenter = True

doCompact = False
if '-compact' in sys.argv:
    doCompact = True

reportAvg = False
if '-report' in sys.argv:
    reportAvg = True

mysom = SOM(initialFile=somfile)
startmap = mysom.readMap(startmapfile)

if doThreshold:
    topUnits = mysom.thresholdMapUnits(startmap, threshold)
    keepTop = len(topUnits)
    print "%d units passing threshold %f will be used" % (keepTop, threshold)
else:
    keepTop = count
    topUnits = mysom.topMapUnits(startmap, keepTop)

numUnits = float(len(mysom.units))
halfRange = numUnits / 2.
if useRank:
    rankMap = mysom.rankWeights()

winnerDict = mysom.readScoreFile(scorefile)

outfile = open(outfilename,'w')
if putHeader:
    dataHeaders = mysom.getDatasetHeader(datafile)
    if doCompact:
        outfile.write('#unit')
    else:
        outfile.write('#map_score\tunit\tcount\tsize_kb')
    for field in dataHeaders:
        outfile.write('\t%s' % field)
    outfile.write('\n')

flatArray = []
boxArray = []
numDim = mysom.inputDim
totalCount = 0
totalSize = 0
averageWeights = [0.] * mysom.inputDim
for unit in topUnits:
    unitCount = len(winnerDict[unit])
    unitSize = 0.
    (row, col) = unit
    for region in winnerDict[unit]:
        (chrom, coord) = region.split(':')
        (start, stop) = coord.split('-')
        unitSize += abs(int(stop) - int(start))/1000.

    #print "unit", unit
    #print "count %d size %.3f" % (unitCount, unitSize)
    #print startmap[row][col]
    #print mysom.weightsArray[row][col]
    totalCount += unitCount
    totalSize += unitSize
    if doCompact:
        outfile.write('unit_%s_%s' % (row, col))
    else:
        outfile.write('%.6f\t' % startmap[row][col])
        outfile.write('(%s,%s)\t%d\t%.3f' % (row, col, unitCount, unitSize))
    currentDim = 0
    displayArray = []
    for val in mysom.weightsArray[row][col]:
        if useRank:
            unitDimRank = rankMap[row][col][currentDim]
            if doRecenter:
                value = (halfRange - unitDimRank) / halfRange
                outfile.write('\t%.4f' % value)
                averageWeights[currentDim] += value
            else:
                outfile.write('\t%d' % unitDimRank)
                averageWeights[currentDim] += unitDimRank
            displayArray.append((numUnits - rankMap[row][col][currentDim]) / numUnits)
        else:
            outfile.write('\t%.6f' % val)
            displayArray.append(val)
            averageWeights[currentDim] += val
        currentDim += 1
    flatArray.append(displayArray)
    outfile.write('\n')

outline = '##avg_weights'
for currentDim in range(mysom.inputDim):
    currentWeight = averageWeights[currentDim] / float(keepTop)
    outline += '\t%.6f' % currentWeight
if reportAvg:
    outfile.write(outline + '\n')

if reportAvg:
    print "\nSummary"
    print "total count = ", totalCount
    print "total size = ", totalSize
    
    print "avg per-unit count = %.2f" % (totalCount / keepTop)
    print "avg per-unit size = %.3f" % (totalSize / keepTop)
    print "avg per-segment size = %.3f" % (totalSize / totalCount)
    outfile.write('##total count = %d\ttotal size = %.3f\tavg count = %.2f\t avg unit size = %.3f\tavg segment size = %.3f\n' % (totalCount, totalSize, totalCount / keepTop, totalSize / keepTop, totalSize / totalCount))
outfile.close()

figure(figsize=(length, width))
#myaxe = axes([0.3, 0.1, 0.55, 0.75])

flatArray.reverse()
Z = array(flatArray)
if useRank:
    c = pcolor(Z, vmin=0.0, vmax=1.0)
else:
    c = pcolor(Z)
c.set_linewidth(0.1)
clim(0.0, 1.0)
xlim(0, numDim)
ylim(0, keepTop)
d = colorbar(orientation="vertical", drawedges=False)

savefig(outimagename, dpi=250)
show()

clf()

figure(figsize=(length, width))


