#
#  farPairs.py
#  ENRAGE
#
#  Created by Ali Mortazavi on 7/13/10.
#

try:
    import psyco
    psyco.full()
except:
    pass

from commoncode import *
import sys, time

print "%s: version 1.4" % sys.argv[0]

if len(sys.argv) < 3:
    print "usage: python %s rdsfile outfile bedfile [-verbose] [-cache numPages] [-minDist bp] [-maxDist bp] [-minLinks count] [-label string]" % sys.argv[0]
    print "\tIs both slow and takes up large amount of RAM"
    sys.exit(1)

minDist = 0
rdsfile = sys.argv[1]
outfilename = sys.argv[2]
outbedname = sys.argv[3]

sameChromOnly = False
if '-sameChrom' in sys.argv:
    sameChromOnly = True

doVerbose = False
if '-verbose' in sys.argv:
    doVerbose = True

doCache = False
cachePages = 0
if '-cache' in sys.argv:
    doCache = True
    cachePages =  int(sys.argv[sys.argv.index('-cache') + 1])

minDist = 1000
if '-minDist' in sys.argv:
    minDist = int(sys.argv[sys.argv.index('-minDist') + 1])

maxDist = 500000
if '-maxDist' in sys.argv:
    maxDist = int(sys.argv[sys.argv.index('-maxDist') + 1])

minCount = 2
if '-minCount' in sys.argv:
    minCount = int(sys.argv[sys.argv.index('-minCount') + 1])

label = rdsfile
if '-label' in sys.argv:
    label = sys.argv[sys.argv.index('-label') + 1]

RDS = readDataset(rdsfile, verbose = True, cache=doCache)
rdsChromList = RDS.getChromosomes()

if doVerbose:
    print time.ctime()

distinct = 0
total = 0
outfile = open(outfilename,'w')
outbed = open(outbedname, 'w')
outbed.write('track name="%s distal pairs" color=0,255,0\n' % label)

def splitReadWrite(chrom, numPieces, startList, stopList, rsense, readName, plusSense, minusSense):
    readSizes = '%d' % (stopList[0] - startList[0])
    readCoords = '0'
    #leftStart = startList[0]
    leftStart = startList[0] - 1
    rightStop = stopList[-1]
    for index in range(1, numPieces):
        #readSizes += ',%d' % (stopList[index] - startList[index])
        readSizes += ',%d' % (stopList[index] - startList[index] + 1)
        readCoords += ',%d' % (startList[index] - startList[0])
    if rsense == '+':
        senseCode = plusSense
    else:
        senseCode = minusSense
    outline = '%s\t%d\t%d\t%s\t1000\t%s\t0\t0\t%s\t%d\t%s\t%s\n' % (chrom, leftStart, rightStop, readName, rsense, senseCode, numPieces, readSizes, readCoords)
    outbed.write(outline)

readlen = RDS.getReadSize()
flagDict = {}
for achrom in rdsChromList:
    if achrom == 'chrM':
        continue
    print achrom
    uniqDict = RDS.getReadsDict(fullChrom=True, chrom=achrom, noSense=True, withFlag=True, withPairID=True, doUniqs=True, readIDDict=True)
    if doVerbose:
        print len(uniqDict), time.ctime()    
    
    for readID in uniqDict:
        readList = uniqDict[readID]
        if len(readList) == 2:
            total += 1
            (start1, flag1, pair1) = readList[0]
            (start2, flag2, pair2) = readList[1]
            
            if flag1 != flag2:
                dist = abs(start1 - start2)
                startList = [start1, start2]
                stopList = [start1 + readlen, start2 + readlen]
                startList.sort()
                stopList.sort()
                if flag1 != '' and flag2 != '' and minDist < dist < maxDist:
                    nonPCRdup = True
                    if doVerbose:
                        print flag1, flag2, dist
                    mark1 = (flag2, start1, start2)
                    mark2 = (flag1, start1, start2)
                    try:
						if mark1 not in flagDict[flag1]:
							flagDict[flag1].append(mark1)
						else:
							nonPCRdup = False
                    except:
                        flagDict[flag1] = [mark1]
                    try:
						if mark2 not in flagDict[flag2]:
							flagDict[flag2].append(mark2)
						else:
							nonPCRdup = False
                    except:
                        flagDict[flag2] = [mark2]
                    if nonPCRdup:
						splitReadWrite(achrom, 2, startList, stopList, '+', readID, '0,255,0', '0,255,0')

print "%d connected regions" % len(flagDict)

acceptedCount = 0
for region in flagDict:
    flagDict[region].sort()
    regionConnections = {}
    for (region2, start1, start2) in flagDict[region]:
        try:
            regionConnections[region2] += 1
        except:
            regionConnections[region2] = 1
    for region2 in regionConnections:
        if regionConnections[region2] >= minCount:
            acceptedCount += 1
            outfile.write('%s\t%s\t%d\n' % (region, region2, regionConnections[region2]))
            if doVerbose:
                print "%s\t%s\t%d" % (region, region2, regionConnections[region2])

print "%d accepted linkages" % acceptedCount

outfile.close()
outbed.close()

if doVerbose:
    print "finished: ", time.ctime()    
